# Load necessary libraries
library(dynRB)      # For dynamic range box calculations
library(ggplot2)    # For generating plots
library(RColorBrewer) # For color schemes

# Example on how to use dynRB package for Dynamic Range Box calculations

# Read the dataset from a MATLAB file (replace with actual path and file)
finchlist <- readMat("/path/to/your/data/combnewStitchCombo.mat")
finchlist <- as.data.frame(finchlist) # Convert the list to a dataframe

# Select a subset of the data for demonstration
# In a real analysis, ensure this subset is representative or covers the needed variability
trunc <- finchlist[1:100, 1:5]  # Using the first 100 rows and 5 columns

# Manually modify the second row for demonstration purposes
trunc[2, ] <- trunc[1, ]  # Copy the first row to the second
trunc[2, 1] <- 29         # Modify the first element of the second row

# Measure execution time for the dynamic range box calculation
start_time <- Sys.time()
result_dynRB <- dynRB_VPa(trunc)
execution_time <- Sys.time() - start_time
print(paste("Execution time: ", execution_time, " seconds"))

# Process the result to calculate overlaps between clusters
result <- result_dynRB$result 
Overlap <- ifelse(result$V1 == result$V2, NA, result$port_prod) # Assign NA to identical clusters
Overlap <- as.numeric(Overlap)  # Ensure numeric format for overlaps

# Combine results with the overlap data
Result2 <- cbind(result, Overlap) 

# Create a tile plot to visualize the overlaps using ggplot2
theme_set(theme(
  plot.background = element_blank(),
  panel.grid.minor = element_blank(),
  panel.grid.major = element_blank(),
  panel.background = element_blank(),
  panel.border = element_blank(),
  axis.line = element_blank(),
  axis.ticks = element_blank(),
  axis.text.x = element_text(colour = "black", size = rel(1.5), angle = 35, hjust = 1),
  axis.text.y = element_text(colour = "black", size = rel(1.5)),
  axis.title.x = element_blank(),
  axis.title.y = element_blank()
))

# Define color gradient for the heatmap
color_gradient <- colorRampPalette(c("white", "navyblue"))

# Plot the heatmap
ggplot(Result2, aes(x = V1, y = V2, fill = Overlap)) + 
  geom_tile(color = "black") +
  scale_fill_gradientn(colours = color_gradient(100), na.value = "lightgrey", 
                       limits = range(Overlap, na.rm = TRUE), 
                       guide = "colorbar") +
  theme_set(theme_change)

# Note: This code is provided as an example to demonstrate the usage of the dynRB package
# and should not be considered as actual analysis.
